--Drawing defaults
minigame.lineWidth = 1
minigame.pointSize = 1
minigame.color = 1
minigame.background = 0

--Translation defaults
minigame.x = 1
minigame.y = 1

--Scissor defaults
minigame.scissorX = 1
minigame.scissorY = 1
minigame.scissorW = pixels
minigame.scissorH = pixels

function minigame.fixedColor(c)
	if type(c) ~= "number" then
		c = 1
	end
	if c > 1 then
		c = 1
	elseif c < 0 then
		c = 0
	end
	return math.floor(c*10/2)*2/10
end

function minigame.draw(imgdata, x, y)
	local t = imageToData(imgdata)
	local nx, ny = minigame.x+x-1, minigame.y+y-1
	for px = 1, #t do
		for py = 1, #t[px] do
			local ax, ay = nx+px-1, ny+py-1
			if minigame.onScissor(ax, ay) then
				if t[px][py] then
					pixel[ax][ay] = math.max(0, t[px][py]-(1-minigame.color))
				end
			end
		end
	end
end

function minigame.rectangle(mode, x, y, width, height)
	local nx, ny = minigame.x, minigame.y
	if mode == "fill" then
		for px = x, x+width-1 do
			for py = y, y+height-1 do
				local ax, ay = nx+px-1, ny+py-1
				if minigame.onScissor(ax, ay) then
					pixel[ax][ay] = minigame.color
				end
			end
		end
	elseif mode == "line" then
		local stx, sty, sts = x, y, minigame.lineWidth
		if sts ~= 1 and math.mod(sts, 2) ~= 0 then
			stx = stx-math.floor((sts-1)/2)
			sty = sty-math.floor((sts-1)/2)
		elseif sts ~= 1 then
			stx = stx-math.floor(sts/2)
			sty = sty-math.floor(sts/2)
		end
		minigame.rectangle("fill", stx, sty, width, sts)
		minigame.rectangle("fill", stx, sty+1, sts, height-2*sts)
		minigame.rectangle("fill", stx+width-1, sty+1, sts, height-2*sts)
		minigame.rectangle("fill", stx, sty+height-1, width, sts)
	end
end

function minigame.circle(mode, x, y, radius)
	
end

function minigame.line(x1, y1, x2, y2)

end

function minigame.polygon( ... )
	local args = { ... }
	local mode = args[1]
end

function minigame.point(x, y)
	local nx, ny = minigame.x+x-1, minigame.y+y-1
	if minigame.onScissor(nx, ny) then
		pixel[nx][ny] = minigame.color
	end
end

function minigame.setLineWidth(n)
	minigame.lineWidth = n or 1
end

function minigame.setPointSize(n)
	minigame.pointSize = n or 1
end

function minigame.print(txt, x, y)
	if type(txt) ~= "string" then
		txt = tostring(txt)
	end
	if type(x) ~= "number" then
		error("\"minigame.print\" argument #2 must be a number")
	end
	if type(y) ~= "number" then
		error("\"minigame.print\" argument #3 must be a number")
	end
	
	local nx, ny = minigame.x, minigame.y
	for letter = 1, string.len(txt) do
		local let = string.sub(txt, letter, letter)
		if let == "|" then
			ny = ny+8
			nx = -(letter*6)+1
		elseif let ~= " " then
			for px = 1, 6 do
				for py = 1, 8 do
					local ax, ay = x+nx+(letter-1)*6+px-3, y+ny+py-2
					if minigame.onScissor(ax, ay) then
						local nlet = let
						if not fontdataquads[nlet] then
							nlet = string.lower(nlet)
						end
						if fontdataquads[nlet] then
							local pix = fontdataquads[let][px][py]
							if pix then
								pixel[ax][ay] = math.max(0, pix-(1-minigame.color))
							end
						end
					end
				end
			end
		end
	end
end

function minigame.setScissor(x, y, width, height)
	if x and y and width and height then
		minigame.scissorX = x
		minigame.scissorY = y
		minigame.scissorW = width
		minigame.scissorH = height
	else
		minigame.scissorX = 1
		minigame.scissorY = 1
		minigame.scissorW = pixels
		minigame.scissorH = pixels
	end
end

function minigame.translate(x, y)
	if x and y then
		minigame.x = x
		minigame.y = y
	else
		minigame.x = 1
		minigame.y = 1
	end
end

function minigame.onScissor(x, y)
	if x >= minigame.scissorX and y >= minigame.scissorY and x <= minigame.scissorX+minigame.scissorW-1 and y <= minigame.scissorY+minigame.scissorH-1 then
		return minigame.onScreen(x, y)
	end
	return false
end

function minigame.onScreen(x, y)
	if x >= 1 and x <= pixels and y >= 1 and y <= pixels then
		return true
	end
	return false
end

function minigame.setColor(color)
	minigame.color = minigame.fixedColor(color)
end

function minigame.setBackground(color)
	if color then
		minigame.background = minigame.fixedColor(color)
	else
		minigame.background = 0
	end
end

function imageToData(imgdata)
	if type(imgdata) == "table" then
		--let's hope it is formatted correctly...
		return imgdata
	end
	local ret = {}
	local retwidth = imgdata:getWidth()
	local retheight = imgdata:getHeight()
	for x = 1, imgdata:getWidth() do
		ret[x] = {}
		for y = 1, imgdata:getHeight() do
			local r, g, b, a = imgdata:getPixel(x-1, y-1)
			local color = (r+g+b)/3
			if a <= 32 then
				ret[x][y] = false
			else
				ret[x][y] = math.floor((255-color)/255*10/2)/10*2
			end
		end
	end
	return ret
end