--[[
	MINIGAME SIMULATOR 0.1
		by HugoBDesigner
	
	Licensed under the Creative Commons 3.0 Share-Alike terms.
		http://creativecommons.org/licenses/by-nc-sa/3.0/
	
	You're entirely free to make/sell/share/do whatever you want with files that run on this game.
	BUT you CANNOT sell/share/do whatever you want with the game itself or with content created
	by someone else without it's author's permission.
	
	╔═════════════════════╗
	║ Version 0.1         ║
	╠═════════════════════╣
	║ Changelog:          ║
	║  - Started project. ║
	╚═════════════════════╝
	╔══════════════════════════════════════════════════════╗
	║ TO DO:                                               ║
	║  - Create game selection menu;                       ║
	║  - Finish drawing functions (circle, polygon, etc.); ║
	║  - Add color blending mode (for transparency);       ║
	║  - Create settings menu;                             ║
	║  - Score and highscore system;                       ║
	║  - Secondary screen (for pause, score, level, etc.); ║
	║  - Game speed and level settings;                    ║
	║  - Game paused;                                      ║
	║  - Game timer;                                       ║
	║  - Custom error handler;                             ║
	║  - Game sorting and categorization systems;          ║
	║  - Online game downloader;                           ║
	║  - More pre-made games (Tron, Snake and Labyrinth).  ║
	╚══════════════════════════════════════════════════════╝
]]

minigame = {}
function love.load()
	pixels = 32
	require "class"
	require "draw"
	require "menu"
	
	fontglyphs = "abcdefghijklmnopqrstuvwxyz0123456789.!?#&()[]{}" .. "LRUDOXS" .. "ABCDEFG"
	--	Upper cases are:	► ◄ ▲ ▼ ○ × □		:) :( :D :o :| ;) :P
	games = love.filesystem.getDirectoryItems("games") --All the games you can have
	table.sort(games)
	
	quickTest = "snake"
	
	state = "menu"
	currentgame = false
	gamelevel = 1
	gamespeed = 1
	gamelives = 1
	gamescore = 0
	gamehighscore = 0
	
	font = love.graphics.newFont(12)
	
	if love.filesystem.exists("highscore.lua") then
		local txt = love.filesystem.read("highscore.lua")
		if tonumber(txt) then
			gamehighscore = tonumber(txt)
		end
	end
	
	local delete = {}
	if #games >= 1 then
		for i = 1, #games do
			if string.sub(games[i], -4, -1) == ".lua" or string.sub(games[i], -4, -1) == ".txt" then
				local current = string.sub(games[i], 1, -5)
				if #games >= i + 1 then
					while string.sub(games[i+1], 1, -5) == current do
						table.remove(games[i+1])
					end
				end
				games[i] = current
			else
				table.insert(delete, i, 1)
			end
		end
	else
		games = {}
	end
	for i, v in ipairs(delete) do
		table.remove(games, v)
	end
	mkfile()
	scale = 1 --do not change. Used for technical purposes
	love.graphics.setBackgroundColor(0, 0, 0, 255)
	love.graphics.setDefaultFilter("nearest", "nearest")
	
	gamewidth = 300
	gameheight = 600
	love.window.setMode(gamewidth*scale, gameheight*scale)
	love.window.setTitle("Minigame")
	
	----------
	--IMAGES--
	----------
	minigame.skin = "default"
	loadSkin()
	
	fontquads = {}
	fontdataquads = {}
	for i = 1, fontimg:getWidth()/6 do
		table.insert(fontquads, love.graphics.newQuad((i-1)*6, 0, 6, 8, fontimg:getWidth(), fontimg:getHeight()))
		local newdata = love.image.newImageData(6, 8)
		for x = 0, 5 do
			local nx = x+(i-1)*6
			for y = 0, 7 do
				newdata:setPixel(x, y, fontdata:getPixel(nx, y))
			end
		end
		fontdataquads[string.sub(fontglyphs, i, i)] = imageToData(newdata)
	end
	
	-----------
	--BUTTONS--
	-----------
	
	buttonU = {k1 = "up", k2 = "w", j = 0}		--UP
	buttonD = {k1 = "down", k2 = "s", j = 0}	--DOWN
	buttonL = {k1 = "left", k2 = "a", j = 0}	--LEFT
	buttonR = {k1 = "right", k2 = "d", j = 0}	--RIGHT
	buttonB = {k1 = " ", k2 = "enter", j = 0}	--BIG
	buttonP = {k1 = "p", k2 = "pause", j = 0}	--PAUSE
	
	--		   		 mouse	    keyboard 1  keyboard 2  joystick
	buttonU.state = {m = false, k1 = false, k2 = false, j = false} --UP
	buttonD.state = {m = false, k1 = false, k2 = false, j = false} --DOWN
	buttonL.state = {m = false, k1 = false, k2 = false, j = false} --LEFT
	buttonR.state = {m = false, k1 = false, k2 = false, j = false} --RIGHT
	buttonB.state = {m = false, k1 = false, k2 = false, j = false} --BIG
	buttonP.state = {m = false, k1 = false, k2 = false, j = false} --PAUSE
	
	
	buttonData = {
		up = 	{x = 100, y = 410, r = 20},
		down = 	{x = 100, y = 490, r = 20},
		left =	{x =  60, y = 450, r = 20},
		right = {x = 140, y = 450, r = 20},
		big =	{x = 230, y = 450, r = 30},
		pause =	{x = 230, y = 360, r = 10},
		U = false,
		D = false,
		L = false,
		R = false,
		B = false,
		P = false
	}
	
	testmode = false
	gamepaused = false
	minigamecolor = {0,205,255}
	
	lastlpress = 0
	lastrpress = 0
	
	quads = {}
	pixel = {}
	
	pixels = 32
	
	for x = 1, pixels do
		quads[x] = {}
		pixel[x] = {}
		for y = 1, pixels do
			pixel[x][y] = false
			quads[x][y] = love.graphics.newQuad((x-1)*5+x, (y-1)*5+y, 5, 5, 193, 193)
		end
	end
	
	--intro()
	
	if quickTest then
		loadGame(quickTest)
	else
		_G[state .. "_load"]()
	end
end

function loadGame(game)
	currentgame = game
	if _G[currentgame .. "_load"] then
		_G[currentgame .. "_load"](gamelevel)
		print("current game: "..currentgame)
	end
	state = "game"
end

function love.update(dt)
	dt = math.min(dt, 1/60)
	if state == "game" then
		if _G[currentgame .. "_update"] then
			_G[currentgame .. "_update"](dt*gamespeed)
		end
	else
		_G[state .. "_update"](dt)
	end
	
	if testmode then
		if love.mouse.isDown("l") then
			local p = 193/32
			local x = love.mouse.getX()
			local y = love.mouse.getY()
			local nx = math.floor((x-54)/p/scale)
			local ny = math.floor((y-54)/p/scale)
			if nx >= 0 and nx <= 31 and ny >= 0 and ny <= 31 then
				if not pixel[nx+1][ny+1] then
					pixel[nx+1][ny+1] = true
				end
			end
		end
		
		if love.mouse.isDown("r") then
			local p = 193/32
			local x = love.mouse.getX()
			local y = love.mouse.getY()
			local nx = math.floor((x-54)/p/scale)
			local ny = math.floor((y-54)/p/scale)
			if nx >= 0 and nx <= 31 and ny >= 0 and ny <= 31 then
				if pixel[nx+1][ny+1] then
					pixel[nx+1][ny+1] = false
				end
			end
		end
	end
	
	--reset screen so nothing stays
	minigame.setColor(1)
	for x = 1, pixels do
		for y = 1, pixels do
			pixel[x][y] = 0
		end
	end
	
	--Draw background
	minigame.setColor(minigame.background)
	minigame.rectangle("fill", 1, 1, 32, 32)
	
	--Draw game
	if state == "game" and currentgame then
		if _G[currentgame .. "_draw"] then
			_G[currentgame .. "_draw"]()
		end
	end
	
	if not (love.mouse.isDown("l") or love.mouse.isDown("r")) then
		buttonU.state.m = false
		getButton("up", "released")
		buttonD.state.m = false
		getButton("down", "released")
		buttonL.state.m = false
		getButton("left", "released")
		buttonR.state.m = false
		getButton("right", "released")
		buttonB.state.m = false
		getButton("big", "released")
		buttonP.state.m = false
		getButton("pause", "released")
	end
end

function love.draw()
	local r, g, b = unpack(minigamecolor)
	love.graphics.setColor(r, g, b, 255)
	love.graphics.draw(minigameimg, 0, 0, 0, scale, scale)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(minigameoverlayimg, 0, 0, 0, scale, scale)
	love.graphics.setColor(0, 0, 0, 105)
	if buttonData.U then
		love.graphics.circle("fill", buttonData.up.x, buttonData.up.y, buttonData.up.r, 32)
	end
	if buttonData.D then
		love.graphics.circle("fill", buttonData.down.x, buttonData.down.y, buttonData.down.r, 32)
	end
	if buttonData.L then
		love.graphics.circle("fill", buttonData.left.x, buttonData.left.y, buttonData.left.r, 32)
	end
	if buttonData.R then
		love.graphics.circle("fill", buttonData.right.x, buttonData.right.y, buttonData.right.r, 32)
	end
	if buttonData.B then
		love.graphics.circle("fill", buttonData.big.x, buttonData.big.y, buttonData.big.r, 32)
	end
	if buttonData.P then
		love.graphics.circle("fill", buttonData.pause.x, buttonData.pause.y, buttonData.pause.r, 32)
	end
	
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(backimg, 54, 54, 0, scale, scale)
	if testmode then
		love.graphics.draw(gridimg, 54, 54, 0, scale, scale)
	end
	
	if state then
		--_G[state .. "_draw"]()
	end
	
	for x = 1, pixels do
		for y = 1, pixels do
			if pixel[x][y] ~= 0 then
				love.graphics.setColor(255, 255, 255, 255*pixel[x][y])
				love.graphics.draw(frontimg, quads[x][y], ((x-1)*5+x)*scale+54*scale, ((y-1)*5+y)*scale+54*scale, 0, scale, scale)
			end
		end
	end
	
	love.graphics.setColor(0, 0, 0, 255)
	if currentgame then
		love.graphics.print("Current game: " .. currentgame, 5, 5)
	else
		love.graphics.print("Current game: none", 5, 5)
	end
	
	local txt = "Score = " .. gamescore
	love.graphics.print(txt, gamewidth-5-font:getWidth(txt), 5)
	txt = "Highscore = " .. gamehighscore
	love.graphics.print(txt, gamewidth-5-font:getWidth(txt), 5+font:getHeight())
	txt = "Lives = " .. gamelives
	love.graphics.print(txt, gamewidth-5-font:getWidth(txt), 5+font:getHeight()*2)
end

function love.keypressed(key, unicode)
	if key == buttonU.k1 or key == buttonU.k2 then
		getButton("up", "pressed")
		if key == buttonU.k1 then
			buttonU.state.k1 = true
		else
			buttonU.state.k2 = true
		end
	elseif key == buttonD.k1 or key == buttonD.k2 then
		getButton("down", "pressed")
		if key == buttonD.k1 then
			buttonD.state.k1 = true
		else
			buttonD.state.k2 = true
		end
	elseif key == buttonL.k1 or key == buttonL.k2 then
		getButton("left", "pressed")
		if key == buttonL.k1 then
			buttonL.state.k1 = true
		else
			buttonL.state.k2 = true
		end
	elseif key == buttonR.k1 or key == buttonR.k2 then
		getButton("right", "pressed")
		if key == buttonR.k1 then
			buttonR.state.k1 = true
		else
			buttonR.state.k2 = true
		end
	elseif key == buttonB.k1 or key == buttonB.k2 then
		getButton("big", "pressed")
		if key == buttonB.k1 then
			buttonB.state.k1 = true
		else
			buttonB.state.k2 = true
		end
	elseif key == buttonP.k1 or key == buttonP.k2 then
		getButton("pause", "pressed")
		if key == buttonP.k1 then
			buttonP.state.k1 = true
		else
			buttonP.state.k2 = true
		end
	end

	if key == "-" then
		if currentgame > 1 then
			currentgame = currentgame - 1
		elseif #games >= 1 then
			currentgame = #games
		end
	end
	
	if key == "=" then
		if #games >= 1 then
			if currentgame < #games then
				currentgame = currentgame + 1
			else
				currentgame = 1
			end
		end
	end
	
	if key == "m" then
		currentgame = 0
	end
	
	if key == "escape" then
		love.event.quit()
	end
	
	if key == "t" then
		testmode = not testmode
	end
	
	if key == "s" and testmode then
		local text = ""
		for x = 1, 32 do
			for y = 1, 32 do
				if x == 1 and y == 1 then
					text = "{" .. tostring(pixel[x][y])
				elseif x < 32 and y < 32 then
					text = text .. ", " .. tostring(pixel[x][y])
				else
					text = text .. ", " .. tostring(pixel[x][y]) .. "}"
				end
			end
		end
		
		local number = 1
		local filename = "0001"
		while love.filesystem.exists("tests/" .. filename .. ".txt") do
			local z = "000"
			number = number + 1
			if number >= 10 and number < 100 then
				z = "00"
			elseif number >= 100 and number < 1000 then
				z = "0"
			elseif number > 1000 then
				z = ""
			end
			filename = z .. tostring(number)
		end
		love.filesystem.mkdir("tests")
		love.filesystem.write("tests/" .. filename .. ".txt", text)
	end
end

function love.keyreleased(key, unicode)
	if key == buttonU.k1 or key == buttonU.k2 then
		if key == buttonU.k1 then
			buttonU.state.k1 = false
		else
			buttonU.state.k2 = false
		end
		getButton("up", "released")
	elseif key == buttonD.k1 or key == buttonD.k2 then
		if key == buttonD.k1 then
			buttonD.state.k1 = false
		else
			buttonD.state.k2 = false
		end
		getButton("down", "released")
	elseif key == buttonL.k1 or key == buttonL.k2 then
		if key == buttonL.k1 then
			buttonL.state.k1 = false
		else
			buttonL.state.k2 = false
		end
		getButton("left", "released")
	elseif key == buttonR.k1 or key == buttonR.k2 then
		if key == buttonR.k1 then
			buttonR.state.k1 = false
		else
			buttonR.state.k2 = false
		end
		getButton("right", "released")
	elseif key == buttonB.k1 or key == buttonB.k2 then
		if key == buttonB.k1 then
			buttonB.state.k1 = false
		else
			buttonB.state.k2 = false
		end
		getButton("big", "released")
	elseif key == buttonP.k1 or key == buttonP.k2 then
		if key == buttonP.k1 then
			buttonP.state.k1 = false
		else
			buttonP.state.k2 = false
		end
		getButton("pause", "released")
	end
end

function love.mousepressed(x, y, button)
	if button == "l" or button == "r" then
		--Up button
		if getRadius(buttonData.up.x*scale, buttonData.up.y*scale, x, y, buttonData.up.r*scale) then
			getButton("up", "pressed")
			buttonU.state.m = true
		end
		
		--Down button
		if getRadius(buttonData.down.x*scale, buttonData.down.y*scale, x, y, buttonData.down.r*scale) then
			getButton("down", "pressed")
			buttonD.state.m = true
		end
		
		--Left button
		if getRadius(buttonData.left.x*scale, buttonData.left.y*scale, x, y, buttonData.left.r*scale) then
			getButton("left", "pressed")
			buttonL.state.m = true
		end
		
		--Right button
		if getRadius(buttonData.right.x*scale, buttonData.right.y*scale, x, y, buttonData.right.r*scale) then
			getButton("right", "pressed")
			buttonR.state.m = true
		end
		
		--Big button
		if getRadius(buttonData.big.x*scale, buttonData.big.y*scale, x, y, buttonData.big.r*scale) then
			getButton("big", "pressed")
			buttonB.state.m = true
		end
		
		--Pause button
		if getRadius(buttonData.pause.x*scale, buttonData.pause.y*scale, x, y, buttonData.pause.r*scale) then
			getButton("pause", "pressed")
			buttonP.state.m = true
		end
	end
end

function love.mousereleased(x, y, button)
	if button == "l" or button == "r" then
		--Up button
		if getRadius(100*scale, 410*scale, x, y, 20*scale) then
			buttonU.state.m = false
			getButton("up", "released")
		end
		
		--Down button
		if getRadius(100*scale, 490*scale, x, y, 20*scale) then
			buttonD.state.m = false
			getButton("down", "released")
		end
		
		--Left button
		if getRadius(60*scale, 450*scale, x, y, 20*scale) then
			buttonL.state.m = false
			getButton("left", "released")
		end
		
		--Right button
		if getRadius(140*scale, 450*scale, x, y, 20*scale) then
			buttonR.state.m = false
			getButton("right", "released")
		end
		
		--Big button
		if getRadius(230*scale, 450*scale, x, y, 30*scale) then
			buttonB.state.m = false
			getButton("big", "released")
		end
		
		--Big button
		if getRadius(230*scale, 360*scale, x, y, 10*scale) then
			buttonP.state.m = false
			getButton("pause", "released")
		end
	end
end

function mkfile()
	local fulltext = ""
	if #games >= 1 then
		for i = 1, #games do
			local text = ""
			if love.filesystem.exists("games/" .. games[i] .. ".lua") then
				text = love.filesystem.read("games/" .. games[i] .. ".lua")
			elseif love.filesystem.exists("games/" .. games[i] .. ".txt") then
				text = love.filesystem.read("games/" .. games[i] .. ".txt")
			end
			fulltext = fulltext .. games[i] .. "_data = {}\r\nlocal self = " .. games[i] .. "_data\r\n"
			
			text = tostring(text)
			
			if i == 1 then
				fulltext = fulltext .. text .. "\r\n"
			else
				fulltext = fulltext .. "\r\n\r\n--==============================--\r\n\r\n" .. text .. "\r\n"
			end
		end
		love.filesystem.write("allgames.lua", fulltext)
		require "allgames"
	end
end

function string:split(delimiter) --Not by me
	local result = {}
	local from	= 1
	local delim_from, delim_to = string.find( self, delimiter, from	 )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from  )
	end
	table.insert( result, string.sub( self, from  ) )
	return result
end

function dist(x1, y1, x2, y2)
	local w, h
	if x1 == x2 then w = 0 end
	if y1 == y2 then h = 0 end
	if x1 > x2 then w = x1 - x2 end
	if x1 < x2 then w = x2 - x1 end
	if y1 > y2 then h = y1 - y2 end
	if y1 < y2 then h = y2 - y1 end
	return math.sqrt(w*w + h*h)
end

function getImage(file)
	local imgdt = love.image.newImageData(file)
	local points = {}
	local w, h = imgdt:getWidth(), imgdt:getHeight()
	
	for x = 1, 32 do
		points[x] = {}
		for y = 1, 32 do
			points[x][y] = false
		end
	end
	
	for x = 0, w-1 do
		for y = 0, h-1 do
			local r, g, b, a = imgdt:getPixel(x, y)
			if a > 0 then
				points[x+1][y+1] = true
			end
		end
	end
	
	return points
end

function getButton(button, mode)
	local b = ""
	if button == "up" then b = "U" end
	if button == "down" then b = "D" end
	if button == "left" then b = "L" end
	if button == "right" then b = "R" end
	if button == "big" then b = "B" end
	if button == "pause" then b = "P" end
	
	if mode == "pressed" then
		local can = true
		if _G["button" .. b].state.m == true then
			can = false
		elseif _G["button" .. b].state.k1 == true then
			can = false
		elseif _G["button" .. b].state.k2 == true then
			can = false
		elseif _G["button" .. b].state.j == true then
			can = false
		end
		
		if can then
			buttonPressed(b)
		end
	elseif mode == "Released" or mode == "released" then
		local can = true
		if _G["button" .. b].state.m == true then
			can = false
		elseif _G["button" .. b].state.k1 == true then
			can = false
		elseif _G["button" .. b].state.k2 == true then
			can = false
		elseif _G["button" .. b].state.j == true then
			can = false
		end
		
		if can then
			buttonReleased(b)
		end
	end
end

function buttonPressed(button)
	if state == "game" then
		if _G[currentgame .. "_buttonPressed"] then
			_G[currentgame .. "_buttonPressed"](button)
		end
	else
		_G[state .. "_buttonPressed"](button)
	end
	buttonData[button] = true
end

function buttonReleased(button)
	if state == "game" then
		if _G[currentgame .. "_buttonReleased"] then
			_G[currentgame .. "_buttonReleased"](button)
		end
	else
		_G[state .. "_buttonReleased"](button)
	end
	buttonData[button] = false
end

function getRadius(ox, oy, px, py, r)
	if dist(ox, oy, px, py) <= r then
		return true
	else
		return false
	end
end

function table.contains(t, e)
	for i, v in pairs(t) do
		if v == e then
			return i
		else
			if type(v) == "table" and type(e) == "table" then
				local can = true
				for a, b in pairs(v) do
					if e[a] ~= v[a] then
						can = false
						break
					end
				end
				if can then return i end
			end
		end
	end
	return false
end

function table.copy(t)
	local ret = {}
	for i, v in pairs(t) do
		if type(v) == "table" then
			ret[i] = table.copy(v)
		else
			ret[i] = v
		end
	end
	return ret
end

function round(n, d)
	local d = d or 2
	return math.floor(n*10^d)/10^d
end

function string.split(s, d)
	local t = {}
	local pos = 1
	for i = 1, string.len(s) do
		local c = string.sub(s, i, i)
		if not t[pos] then
			t[pos] = ""
		end
		if c == d then
			pos = #t+1
		else
			t[pos] = t[pos] .. c
		end
	end
	return t
end

function loadSkin()
	local sk = minigame.skin or "default"
	if love.filesystem.exists("graphics/"..sk.."/back.png") then
		backimg = love.graphics.newImage("graphics/"..sk.."/back.png")
	else
		backimg = love.graphics.newImage("graphics/default/back.png")
	end
	if love.filesystem.exists("graphics/"..sk.."/front.png") then
		frontimg = love.graphics.newImage("graphics/"..sk.."/front.png")
	else
		frontimg = love.graphics.newImage("graphics/default/front.png")
	end
	if love.filesystem.exists("graphics/"..sk.."/minigame.png") then
		minigameimg = love.graphics.newImage("graphics/"..sk.."/minigame.png")
	else
		minigameimg = love.graphics.newImage("graphics/default/minigame.png")
	end
	if love.filesystem.exists("graphics/"..sk.."/minigame_overlay.png") then
		minigameoverlayimg = love.graphics.newImage("graphics/"..sk.."/minigame_overlay.png")
	else
		minigameoverlayimg = love.graphics.newImage("graphics/default/minigame_overlay.png")
	end
	if love.filesystem.exists("graphics/"..sk.."/font.png") then
		fontimg = love.graphics.newImage("graphics/"..sk.."/font.png")
		fontdata = love.image.newImageData("graphics/"..sk.."/font.png")
	else
		fontimg = love.graphics.newImage("graphics/default/font.png")
		fontdata = love.image.newImageData("graphics/default/font.png")
	end
end

------------------------
-- Minigame functions --
------------------------

function minigame.addScore(n)
	n = math.max(0, n)
	n = math.min(1000000, n)
	n = math.floor(n/100)*100 --only multiples of 100
	gamescore = gamescore + n
	gamescore = math.min(gamescore, 1000000)
	updateHighscore()
end

function minigame.removeScore(n)
	n = math.max(0, n)
	n = math.min(1000000, n)
	n = math.floor(n/100)*100 --only multiples of 100
	gamescore = gamescore - n
	gamescore = math.max(gamescore, 0)
	updateHighscore()
end

function minigame.setScore(n)
	n = math.max(0, n)
	n = math.min(1000000, n)
	n = math.floor(n/100)*100 --only multiples of 100
	gamescore = n
	gamescore = math.max(gamescore, 0)
	gamescore = math.min(gamescore, 1000000)
	updateHighscore()
end

function updateHighscore()
	if gamehighscore < gamescore then
		gamehighscore = gamescore
		love.filesystem.write("highscore.lua", tostring(gamehighscore))
	end
end

function minigame.resetLevel()
	_G[currentgame .. "_load"](gamelevel)
end

function minigame.levelUp()
	gamelevel = gamelevel+1
	if gamelevel > 10 then
		gamelevel = 1
	end
	_G[currentgame .. "_load"](gamelevel)
end

function minigame.buttonDown(button)
	return buttonData[button] or false
end

function minigame.setLives(n)
	if type(n) == "number" then
		n = math.floor(math.abs(n))
		gamelives = n
	end
	
	if gamelives == 0 then
		minigame.gameOver()
	end
end

function minigame.addLives(n)
	if type(n) == "number" then
		n = math.floor(math.abs(n))
		gamelives = gamelives+n
	end
end

function minigame.removeLives(n)
	if type(n) == "number" then
		n = math.floor(math.abs(n))
		gamelives = math.max(0, gamelives-n)
	end
	
	if gamelives == 0 then
		minigame.gameOver()
	end
end

function minigame.gameOver()
	--make the game return to the menu
	--For now, it'll just restart the game:
	gamelevel = 1
	loadGame(currentgame)
end